MANAGED_BY_ANNOTATION_KEY = 'app.kubernetes.io/managed-by'

GIT_BRANCH_ANNOTATION_KEY = 'kubetools/git_branch'
GIT_TAG_ANNOTATION_KEY = 'kubetools/git_tag'
GIT_COMMIT_ANNOTATION_KEY = 'kubetools/git_commit'

PROJECT_NAME_LABEL_KEY = 'kubetools/project_name'
ROLE_LABEL_KEY = 'kubetools/role'
NAME_LABEL_KEY = 'kubetools/name'
